package com.example.asus.myfirstgame;

import android.app.Activity;
import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.telephony.TelephonyManager;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.DragEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Random;
import java.util.Set;
import java.util.UUID;

public class GamePanel extends SurfaceView implements SurfaceHolder.Callback {

    public static final int WIDTH = 856;
    public static final int HEIGHT = 480;
    public static final int MOVESPEED = -5;
    private static final int LCD_HEIGHT = 64;
    private static final int LCD_WIDTH = 128;
    private MainThread thread;
    private Background bg;
    private static Player player;
    private static Player enemy;
    private ArrayList<Smokepuff> smoke;
    private long smokeStartTime;
    private ArrayList<TopBorder> topborder;
    private ArrayList<BotBorder> botborder;
    private Random rand = new Random();
    private int maxBorderHeight;
    private int minBorderHeight;
    private boolean topDown = true;
    private boolean newGameCreated;
    private Bitmap missileBitmap;

    private Explosion explosion;
    private long startReset;
    private boolean reset;
    private boolean dissapear;
    private boolean started;
    public static int best;

    // Bluetooth stuff
    private BluetoothAdapter BA = null;
    private Set<BluetoothDevice> pairedDevices = null;
    private BluetoothSocket btSocket = null;
    private OutputStream outStream = null;
    private BluetoothSocketListener bluetoothSocketListener;
    private boolean connected = false;

    private static int screenHeight;
    private static int screenWidth;
    private static int lcdHeightRatio;
    private static int lcdWidthRatio;

    public GamePanel(Context context) {
        super(context);

        // add the callback to the surfaceholder to intercept events
        getHolder().addCallback(this);

        // make gamePanel focusable so it can handle events
        setFocusable(true);
        DisplayMetrics displayMetrics = new DisplayMetrics();
        ((Activity) getContext()).getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        screenHeight = displayMetrics.heightPixels;
        screenWidth = displayMetrics.widthPixels;

        lcdHeightRatio = HEIGHT / LCD_HEIGHT;
        lcdWidthRatio = WIDTH / LCD_WIDTH;
    }

    private void connectBTH() {
        /**
         * Get paired devices and popup a menu to select which to connect
         */

        BA = BluetoothAdapter.getDefaultAdapter();
        pairedDevices = BA.getBondedDevices();
        final UUID uuid = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

        ArrayList<String> devicesName = new ArrayList<String>();
        for (BluetoothDevice bt : pairedDevices) {
            devicesName.add(bt.getName());
        }

        final CharSequence[] cs = devicesName.toArray(new CharSequence[devicesName.size()]);

        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle("Select a device");
        builder.setItems(cs, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                for (BluetoothDevice bt : pairedDevices) {
                    BluetoothDevice device = BA.getRemoteDevice(bt.getAddress());
                    if (device.getName().equals(cs[i])) {
                        try {
                            btSocket = device.createRfcommSocketToServiceRecord(uuid);
                            BA.cancelDiscovery();
                            btSocket.connect();
                            outStream = btSocket.getOutputStream();
                            bluetoothSocketListener = new BluetoothSocketListener(btSocket);

                            connected = true;
                        } catch (Exception e) {
                            Log.d("eroare", "conexiune bluetooth");
                        }
                    }
                }
            }
        });
        builder.show();
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        if (!player.getPlaying() || !enemy.getPlaying()) {
            if (event.getAction() == MotionEvent.ACTION_DOWN) {
                if ((!player.getPlaying() || !enemy.getPlaying()) && newGameCreated && reset) {
                    player.setPlaying(true);
                    enemy.setPlaying(true);
                }
                if (player.getPlaying()) {
                    if (!started) started = true;
                    reset = false;
                }
                return true;
            }
            if (event.getAction() == MotionEvent.ACTION_UP) {
                return true;
            }
        } else {
            if (event.getAction() == MotionEvent.AXIS_Y) {
                if (event.getY() - player.y < 0) {
                    player.dy -= 0.3f;
                }
                else if (event.getY() - player.y > 0) {
                    player.dy += 0.3f;
                }

                int posX = (int)player.getX() / lcdWidthRatio;
                int posY = (int)player.getY() / lcdHeightRatio;

                String serialMsg = "1|" + posX + "|" + posY + ",";

                try {
                    if (outStream != null) {
                        Log.d("write string", serialMsg);
                        outStream.write(serialMsg.getBytes());
                    } else {
                        Log.d("eroare outStream", "outStream e null");
                    }
                } catch (Exception e) {
                    Log.d("eroare", "scriere bluetooth");
                }
            }

            player.addMissile(missileBitmap);
            return true;
        }

        return true;
    }

    @Override
    public void surfaceCreated(SurfaceHolder holder) {

        bg = new Background(BitmapFactory.decodeResource(getResources(), R.drawable.grassbg1));
        player = new Player(BitmapFactory.decodeResource(getResources(), R.drawable.helicopter), 65, 25, 3);

        Bitmap enemyBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.enemy);
        enemy = new Player(enemyBitmap, 65, 25, 3);
        enemy.setX(WIDTH - 130);

        missileBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.missile);
        smoke = new ArrayList<>();
        topborder = new ArrayList<>();
        botborder = new ArrayList<>();

        smokeStartTime = System.nanoTime();

        connectBTH();

        thread = new MainThread(getHolder(), this);

        //we can safety start the game loop
        thread.setRunning(true);
        thread.start();

        if (bluetoothSocketListener != null)
            bluetoothSocketListener.start();
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        boolean retry = true;
        int counter = 0;
        while (retry && counter < 1000) {
            counter++;
            try {
                thread.setRunning(false);
                thread.join();

                if (bluetoothSocketListener != null) {
                    bluetoothSocketListener.turnOnBreakWhile();
                    bluetoothSocketListener.join();
                }
                retry = false;
                thread = null;
                bluetoothSocketListener = null;
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {

    }

    // bool addMissile | int x position | int y position
    private void parseMessage(char msg[]) {
        String coord = "";

        if (msg[0] == '1')
            enemy.addMissile(missileBitmap);

        for (int i = 2; i < msg.length; i++) {
            if (msg[i] == '|') {
                enemy.setX(Integer.valueOf(coord) * lcdWidthRatio);
                coord = "";
            } else {
                coord += msg[i];
            }
        }
        enemy.setY(Integer.valueOf(coord) * lcdHeightRatio);
    }

    public void update() {

        if (player.getPlaying() && enemy.getPlaying()) {

            // check collisions
            checkCollision(player, enemy);
            checkCollision(enemy, player);

            // bluetooth read
            if (bluetoothSocketListener != null && !bluetoothSocketListener.getUnlockReading()) {
                Log.d("message received", bluetoothSocketListener.getMessage());
                parseMessage(bluetoothSocketListener.getMessage().toCharArray());
                bluetoothSocketListener.switchUnlockReading();
            }

            if (topborder.isEmpty()) {
                player.setPlaying(false);
                return;
            }

            bg.update();
            player.update();
            enemy.update();

            // calculate the threshold of height the border can have based on the score
            // max and min border heart are updated, and the border switched direction
            // when either max or min is met
            int progressDenom = 20;
            maxBorderHeight = 30 + player.getScore() / progressDenom;

            // cap max border height so that borders can only take up a total of 1/2 of the screen
            if (maxBorderHeight > HEIGHT / 4)
                maxBorderHeight = HEIGHT / 4;

            minBorderHeight = 5 + player.getScore() / progressDenom;

            // check top border collision
            for (int i = 0; i < topborder.size(); i++) {
                if (collision(topborder.get(i), player))
                    player.setPlaying(false);
            }

            // check bottom border collision
            for (int i = 0; i < botborder.size(); i++) {
                if (collision(botborder.get(i), player))
                    player.setPlaying(false);
            }

            // update top border
            this.updateTopBorder();

            // add smoke puffs
            long elapsed = (System.nanoTime() - smokeStartTime) / 1000000;
            if (elapsed > 120) {
                smoke.add(new Smokepuff(player.getX(), player.getY() + 10));
                smokeStartTime = System.nanoTime();
            }
            for (int i = 0; i < smoke.size(); i++) {
                smoke.get(i).update();
                if (smoke.get(i).getX() < -10) {
                    smoke.remove(i);
                }
            }

        } else {
            player.resetDY();
            enemy.resetDY();
            if (!reset) {
                newGameCreated = false;
                startReset = System.nanoTime();
                reset = true;
                dissapear = true;

                if (!player.getPlaying())
                    explosion = new Explosion(BitmapFactory.decodeResource(getResources(), R.drawable.explosion), player.getX(),
                        player.getY() - 30, 100, 100, 25);
                else if (!enemy.getPlaying()) {
                    explosion = new Explosion(BitmapFactory.decodeResource(getResources(), R.drawable.explosion), enemy.getX(),
                            enemy.getY() - 30, 100, 100, 25);
                }
            }

            explosion.update();
            long resetElapsed = (System.nanoTime() - startReset) / 1000000;

            if (resetElapsed > 1000 && !newGameCreated) {
                newGame();
            }

            if(best < player.getScore())
                best = player.getScore();
        }
    }

    private static boolean collision(GameObject a, GameObject b) {
        return Rect.intersects(a.getRectangle(), b.getRectangle());
    }

    private static void checkCollision(Player player1, Player player2) {
        // loop through every missile
        for (int i = 0; i < player2.missiles.size(); i++) {
            if (GamePanel.collision(player2.missiles.get(i), player1)) {
                player2.missiles.remove(i);
                player1.setPlaying(false);
                break;
            }
        }
    }


    @Override
    public void draw(Canvas canvas) {
        super.draw(canvas);

        final float scaleFactorX = getWidth() / (WIDTH * 1.f);
        final float scaleFactorY = getHeight() / (HEIGHT * 1.f);

        if (canvas != null) {
            final int savedState = canvas.save();
            canvas.scale(scaleFactorX, scaleFactorY);
            bg.draw(canvas);
            if (!dissapear) {
                player.draw(canvas);
                enemy.draw(canvas);
            }
            // draw smokepuffs
            for (Smokepuff sp : smoke) {
                sp.draw(canvas);
            }

            //draw topborder
            for (TopBorder tb : topborder) {
                tb.draw(canvas);
            }

            // draw explosion
            if (started) {
                explosion.draw(canvas);
            }
            drawText(canvas);
            canvas.restoreToCount(savedState);
        }
    }


    public void updateTopBorder() {
        // every 50 points, insert randomly placed top block that break the pattern
        if (player.getScore() % 50 == 0) {
            topborder.add(new TopBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                    topborder.get(topborder.size() - 1).getX() + 20, 0, (int) ((rand.nextDouble() * (maxBorderHeight)) + 1)));
        }

        for (int i = 0; i < topborder.size(); i++) {
            topborder.get(i).update();
            if (topborder.get(i).getX() < -20) {
                topborder.remove(i);

                // remove element of arraylist, replace if by adding a new one
                if (topborder.get(topborder.size() - 1).getHeight() >= maxBorderHeight) {
                    topDown = false;
                }
                if (topborder.get(topborder.size() - 1).getHeight() <= minBorderHeight) {
                    topDown = true;
                }

                // new border added will have larger height
                if (topDown) {
                    topborder.add(new TopBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                            topborder.get(topborder.size() - 1).getX() + 20, 0,
                            topborder.get(topborder.size() - 1).getHeight() + 1));
                } else {
                    topborder.add(new TopBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                            topborder.get(topborder.size() - 1).getX() + 20, 0,
                            topborder.get(topborder.size() - 1).getHeight() - 1));
                }
            }
        }
    }

    public void newGame() {
        dissapear = false;

        botborder.clear();
        topborder.clear();

        player.clearMissiles();
        enemy.clearMissiles();
        smoke.clear();

        minBorderHeight = 5;
        maxBorderHeight = 30;

        player.resetScore();
        player.resetDY();
        player.setY(HEIGHT / 2);
        enemy.resetDY();
        enemy.setY(HEIGHT / 2);

        if (player.getScore() > best) {
            best = player.getScore();
        }

        // create initial borders
        for (int i = 0; i * 20 < WIDTH + 40; i++) {
            if (i == 0) {
                // first top border create
                topborder.add(new TopBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                        i * 20, 0, 10));
            } else {
                topborder.add(new TopBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                        i * 20, 0, topborder.get(i - 1).getHeight() + 1));
            }
        }

        // initial bottom border
        for (int i = 0; i * 20 < WIDTH + 40; i++) {
            if (i == 0) {
                // first bottom border create
                botborder.add(new BotBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                        i * 20, HEIGHT - minBorderHeight));
            } else {
                botborder.add(new BotBorder(BitmapFactory.decodeResource(getResources(), R.drawable.brick),
                        i * 20, botborder.get(i - 1).getY() - 1));
            }
        }
        newGameCreated = true;
    }

    public void drawText(Canvas canvas) {
        Paint paint = new Paint();
        paint.setColor(Color.WHITE);
        paint.setTextSize(30);
        paint.setTypeface(Typeface.create(Typeface.DEFAULT, Typeface.BOLD));
        canvas.drawText("DISTANCE: " + player.getScore(), 10, HEIGHT - 10, paint);
        canvas.drawText("BEST: " + best, WIDTH - 215, HEIGHT - 10, paint);

        if ((!player.getPlaying() || !enemy.getPlaying()) && newGameCreated && reset) {
            Paint paint1 = new Paint();
            paint1.setTextSize(40);
            paint1.setTypeface(Typeface.create(Typeface.DEFAULT, Typeface.BOLD));
            canvas.drawText("PRESS TO START", WIDTH / 2 - 50, HEIGHT / 2, paint1);

            paint1.setTextSize(20);
            canvas.drawText("PRESS AND HOLD TO GO UP", WIDTH / 2 - 50, HEIGHT / 2 + 20, paint1);
            canvas.drawText("RELEASE TO GO DOWN", WIDTH / 2 - 50, HEIGHT / 2 + 40, paint1);
        }
    }

    public static Player getPlayer() {
        return player;
    }
}
