package com.example.asus.myfirstgame;


import android.graphics.Bitmap;
import android.graphics.Canvas;

import java.util.ArrayList;

public class Player extends GameObject {

    private Bitmap spritesheet;
    private int score;
    private boolean up;
    private boolean playing;
    private Animation animation = new Animation();
    private long startTime;
    public ArrayList<Missile> missiles;
    private long missileStartTime;
    private long missileInterval = 2000;

    public Player(Bitmap res, int w, int h, int numFrames){

        x = 100;
        y = GamePanel.HEIGHT / 2;
        dy = 0;
        score = 0;
        height = h;
        width = w;

        Bitmap[] image = new Bitmap[numFrames];
        spritesheet = res;

        for(int i = 0; i < image.length; i++)
        {
            image[i] = Bitmap.createBitmap(spritesheet, i * width, 0, width, height);
        }

        animation.setFrames(image);
        animation.setDelay(10);
        startTime = System.nanoTime();

        missiles = new ArrayList<>();
        playing = false;
        missileStartTime = System.currentTimeMillis();
    }

    public void setUp(boolean b)
    {
        up = b;
    }

    public void update()
    {
        long elapsed = (System.nanoTime() - startTime) / 1000000;
        if(elapsed > 100)
        {
            score++;
            startTime = System.nanoTime();
        }

        animation.update();

        if(dy > 14) dy = 14;
        if(dy < -14) dy = -14;

        y += dy * 2;

        // update missiles
        for (int i = 0; i < missiles.size(); i++) {
            missiles.get(i).update();

            // remove missile if it is way off the screen
            if (missiles.get(i).getX() < -100) {
                missiles.remove(i);
                break;
            }
        }
    }

    public void addMissile(Bitmap missileBitmap) {
        if (System.currentTimeMillis() - missileStartTime > missileInterval) {
            missiles.add(new Missile(missileBitmap, x, y, 45, 15, 13));
            missileStartTime = System.currentTimeMillis();
            missileInterval--;
        }
    }

    public void clearMissiles() {
        missiles.clear();
    }

    public void draw(Canvas canvas)
    {
        canvas.drawBitmap(animation.getImage(), x, y, null);

        // draw missiles
        for (Missile m : missiles) {
            m.draw(canvas);
        }
    }
    public int getScore(){return  score;}
    public boolean getPlaying(){return playing;}
    public void setPlaying(boolean b){playing = b;}
    public void resetDY(){dy = 0;}
    public void resetScore(){score = 0;}

}
