#include "u8g.h"

#if defined(__AVR__)
#include <avr/interrupt.h>
#include <avr/io.h>
#endif

#include <stdlib.h>
#include <util/delay.h>
#include "usart.h"

#define WIDTH  128
#define HEIGHT 64

#define MAX_MISSILES 10
#define HELICOPTER_SIZE 16

const uint8_t helicopter_bitmap1[] U8G_PROGMEM = {
	0x01, 		// 0000 0001
	0x00,		// 0000 0000
	0x80,		// 1000 0000
	0x60,		// 0110 0000
	0x30,		// 0111 1111
	0x3f,		// 0011 1111
	0x60,		// 0110 0000
	0x80		// 1000 0000
};

const uint8_t helicopter_bitmap2[] U8G_PROGMEM = {
	0xff,		// 1111 1111
	0x18,		// 0001 1000
	0x3c,		// 0011 1100
	0x7c,		// 0111 1100
	0xfe,		// 1111 1110
	0xff,		// 1111 1111
	0x7e,		// 0111 1110
	0x3c		// 0011 1100
};

const uint8_t enemy_bitmap1[] U8G_PROGMEM = {
	0xff,		// 1111 1111
	0x18,		// 0001 1000
	0x3c,		// 0011 1100
	0x3e,		// 0011 1110
	0x7f,		// 0111 1111
	0xff,		// 1111 1111
	0x7e,		// 0111 1110
	0x3c		// 0011 1100
};

const uint8_t enemy_bitmap2[] U8G_PROGMEM = {
	0x00, 		// 0000 0000
	0x00,		// 0000 0000
	0x01,		// 0000 0001
	0x06,		// 0000 0110
	0xfe,		// 1111 1110
	0xfc,		// 1111 1100
	0x06,		// 0000 0110
	0x01		// 0000 0001
};

const uint8_t helicopter_missile_bitmap[] U8G_PROGMEM = {
	0x00,		// 0000 0000
	0x00,		// 0000 0000
	0xfc,		// 1111 1100
	0xfe,		// 1111 1110
	0xfc,		// 1111 1100
	0x00,		// 0000 0000
	0x00,		// 0000 0000
	0x00		// 0000 0000
};

const uint8_t enemy_missile_bitmap[] U8G_PROGMEM = {
	0x00,		// 0000 0000
	0x00,		// 0000 0000
	0x3f,		// 0011 1111
	0x7f,		// 0111 1111
	0x3f,		// 0011 1111
	0x00,		// 0000 0000
	0x00,		// 0000 0000
	0x00		// 0000 0000
};

// structura reprezentand coordonatele de pe LCD
typedef struct {
    int x;
    int y;
    short visible;
} coordinates;

typedef struct {
	coordinates coords;
	coordinates missiles[MAX_MISSILES];
	int nr_missiles;
	short isHelicopter;		// 1 - helicopter, 0 - enemy
} player;

u8g_t u8g;
player helicopter;
player enemy;
int next_move;
short lost;
short won;
int score;
volatile short dataReceived = 0;

#define BUFLEN 50
volatile char buff[BUFLEN];
volatile int buff_i = 0;

float helicopterSpeed = 2.f;

// Bluetooth Receive interrupt
ISR(USART0_RX_vect) {

	if (!dataReceived) {
		buff[buff_i] = UDR0;

		// if we have reached the end of the message
		if (buff[buff_i] == ',') {
			buff[buff_i] = '\0';
			dataReceived = 1;
		} else {
			buff_i++;
		}
	}
}

void sendData(void) {
	char data[50];
	int i = 0;

	data[0] = '0';		// missile added
	data[1] = '|';		// separator

	sprintf(&data[2], "%d|%d,", helicopter.coords.x, helicopter.coords.y);

	while (data[i] != '\0') {
		USART0_transmit(data[i]);
		i++;
	}
}

// Bluetooth Send Interrupt
// ISR(USART0_TX_vect) {
// 	// send helicopter position via bluetooth
// 	//sendData();
// }

// initializarea LCD-ului
void u8g_setup(void) {  
    u8g_InitSPI(&u8g, &u8g_dev_st7920_128x64_sw_spi, PN(1, 5), PN(1, 3), PN(1, 2), U8G_PIN_NONE, U8G_PIN_NONE); 
    u8g_SetRot180(&u8g);
}

// initializare butoanelor si a ledurilor
void io_init(void) {
	// butoane
    DDRB &= ~((1 << PB0) | (1 << PB1) | (1 << PB6) | (1 << PB7) | (1 << PB4));
    PORTB |=  (1 << PB0) | (1 << PB1) | (1 << PB6) | (1 << PB7) | (1 << PB4);
}

// initializarea jocului
void game_init(void) {

	score = 0;
	lost = 0;
    
    helicopter.coords.x = 0;
	helicopter.coords.y = HEIGHT / 2;
	helicopter.isHelicopter = 1;

	enemy.coords.x = WIDTH - HELICOPTER_SIZE;
	enemy.coords.y = HEIGHT / 2;
	enemy.isHelicopter = 0;
}

// desenare pagina initiala a jocului
void drawFirstPage(void) {
    u8g_SetFont(&u8g, u8g_font_gdr9);
	
    u8g_DrawStr(&u8g, 0, 11, " Helicopter Game ");
    u8g_DrawStr(&u8g, 0, 26, " Press right ");
	u8g_DrawStr(&u8g, 0, 41, " button to start ");
	u8g_DrawStr(&u8g, 0, 56, " the game. ");
}

// desenare mesaj dupa finalizarea jocului cu scor si level atins
void drawMessage(int message_type) {
    char buffer[33];
	
	u8g_SetFont(&u8g, u8g_font_gdr9);
	
	// lost message
	if (message_type == 0) {
		u8g_DrawStr(&u8g, 0, 11, " Ohh, you lost!! :(( ");
		u8g_DrawStr(&u8g, 0, 26, " Score: ");
		u8g_DrawStr(&u8g, 42, 26, itoa (score, buffer, 10));
		u8g_DrawStr(&u8g, 0, 41, " Press right ");
		u8g_DrawStr(&u8g, 0, 56, "      button. "); 
	} else if (message_type == 1) {
		u8g_DrawStr(&u8g, 0, 11, " You win! Fatality!!");
		u8g_DrawStr(&u8g, 0, 26, " Score: ");
		u8g_DrawStr(&u8g, 42, 26, itoa (score, buffer, 10));
		u8g_DrawStr(&u8g, 0, 41, " Press right ");
		u8g_DrawStr(&u8g, 0, 56, "      button. "); 
	}
}

void addMissile(player *player) {
	
	player->missiles[player->nr_missiles].x = player->coords.x + HELICOPTER_SIZE;
	player->missiles[player->nr_missiles].y = player->coords.y;
	player->missiles[player->nr_missiles].visible = 1;

	player->nr_missiles = (player->nr_missiles == MAX_MISSILES - 1 ? 0 : player->nr_missiles + 1);
}

void drawHelicopter(int x, int y) {
	u8g_DrawBitmapP(&u8g, x, y, 1, 8, helicopter_bitmap1);
	u8g_DrawBitmapP(&u8g, x+8, y, 1, 8, helicopter_bitmap2);
}

void drawEnemy(int x, int y) {
	u8g_DrawBitmapP(&u8g, x, y, 1, 8, enemy_bitmap1);
	u8g_DrawBitmapP(&u8g, x+8, y, 1, 8, enemy_bitmap2);
}

void drawMissile(int x, int y, player *player) {
	if (player->isHelicopter)
		u8g_DrawBitmapP(&u8g, x, y, 1, 8, helicopter_missile_bitmap);
	else
		u8g_DrawBitmapP(&u8g, x, y, 1, 8, enemy_missile_bitmap);
}

void updateMissiles(player *player) {
	int i;

	for (i = 0; i < MAX_MISSILES; i++) {
		if (player->isHelicopter) {
			if (player->missiles[i].visible)
				player->missiles[i].x++;
		} else {
			if (player->missiles[i].visible)
				player->missiles[i].x--;
		}
	}
}

void drawMap(void) {
	char buffer [33];
	int i;

    u8g_SetFont(&u8g, u8g_font_gdr9);

    // draw helicopter
    drawHelicopter(helicopter.coords.x, helicopter.coords.y);

    // draw enemy
    drawEnemy(enemy.coords.x, enemy.coords.y);

    // update helicopter missiles
    updateMissiles(&helicopter);

    // update enemy missiles
    updateMissiles(&enemy);

    // draw helicopter missiles
    for (i = 0; i < MAX_MISSILES; i++) {
    	if (helicopter.missiles[i].visible)
    		drawMissile(helicopter.missiles[i].x, helicopter.missiles[i].y, &helicopter);
    }

    // draw enemy missiles
    for (i = 0; i < MAX_MISSILES; i++) {
    	if (enemy.missiles[i].visible)
    		drawMissile(enemy.missiles[i].x, enemy.missiles[i].y, &enemy);
    }

    // erase helicopter missiles
    for (i = 0; i < MAX_MISSILES; i++) {
    	if (helicopter.missiles[i].x > WIDTH)
    		helicopter.missiles[i].visible = 0;
    }

    // draw enemy missiles
    for (i = 0; i < MAX_MISSILES; i++) {
    	if (enemy.missiles[i].x < 0)
    		enemy.missiles[i].visible = 0;
    }

    score++;

	// score-ul din coltul stanga sus
	u8g_DrawStr(&u8g, 0, 10, " Score: ");
	u8g_DrawStr(&u8g, 44, 10, itoa (score, buffer, 10));
}

void checkButtons(void) {

	// PB0 apasat -> move right
    if (!(PINB & (1 << PB1))) {
		next_move = 0;
	}
	// PB1 apasat -> move left
	else if (!(PINB & (1 << PB7))) {
		next_move = 1;
	}
	// PB2 apasat -> move up
	else if (!(PINB & (1 << PB6))) {
	 	next_move = 2;
	}
	// PB3 apasat -> move down
	else if (!(PINB & (1 << PB0))) {
		next_move = 3;
	}
	// no move
	else {
		next_move = -1;
	}
}

void updatePlayerPosition(void) {
    int i;
    
    checkButtons();

    if (next_move == 0) {
		helicopter.coords.x += helicopterSpeed;
	} else if (next_move == 1) {
		helicopter.coords.x -= helicopterSpeed;
	} else if (next_move == 2) {
		helicopter.coords.y += helicopterSpeed;
	} else if (next_move == 3) {
		helicopter.coords.y -= helicopterSpeed;
	}
	helicopterSpeed += 0.05f;

	if (helicopter.coords.x <= 0 || helicopter.coords.y <= 0 || 
		helicopter.coords.x >= WIDTH || helicopter.coords.y >= HEIGHT) {
		lost = 1;
    }
	
	for (i = 0; i < enemy.nr_missiles; i++) {
		if (enemy.missiles[i].visible &&
			abs(enemy.missiles[i].x - helicopter.coords.x) < 16) {
			lost = 1;
		}
	}

	for (i = 0; i < helicopter.nr_missiles; i++) {
		if (helicopter.missiles[i].visible &&
			abs(helicopter.missiles[i].x - enemy.coords.x) < 16) {
			won = 1;
		}
	}

	// check for received data
	if (dataReceived) {
		parseData(buff);
		buff_i = 0;
	}
}

void parseData(char data[]) {
	int x = 0, y = 0;
	int i;

	for (i = 2; i < buff_i; i++) {
		if (data[i] != '|') {
			x = x * 10 + (data[i] - '0');
		} else break;
	}

	i++;

	while (i < buff_i) {
		y = y * 10 + (data[i] - '0');
		i++;
	}

	enemy.coords.x = x + 90;
	enemy.coords.y = y;

	if (data[0] == '1')
		addMissile(&enemy);

	dataReceived = 0;
}

void checkFireButton(void) {
	if (!(PINB & (1 << PB4))) {
		addMissile(&helicopter);
	}
}

int main(void) {

    u8g_setup();
	io_init();

	sei();
	USART0_init();

	game_init();

	while (1) {
		
		// right button
		if (!(PINB & (1 << PB1))) {
			break;
		}
		u8g_FirstPage(&u8g);
		do {
			drawFirstPage();
		} while (u8g_NextPage(&u8g));
	}

	while (1) {
		
		lost = 0;
		won = 0;
		score = 0;

		// game screen
		while (1) {
			updatePlayerPosition();
			checkFireButton();
			
			u8g_FirstPage(&u8g);
			do {
				drawMap();
			} while (u8g_NextPage(&u8g));
		
			if (lost || won)
				break;
		}
		
		// restart screen
		while (1) { 
			
			// right button
			if (!(PINB & (1 << PB1))) {
				lost = 0;
				won = 0;
				break;
			}
				 
			u8g_FirstPage(&u8g);
			do {
				if (lost) {
					drawMessage(0);
				} else if (won) {
					drawMessage(1);
				}
			} while (u8g_NextPage(&u8g));
		}
	}
}
